@@ -4,7 +4,7 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -49,6 +49,8 @@ static gboolean status_flag;
 static gboolean enable_3gpp_flag;
 static gboolean disable_3gpp_flag;
 static gboolean get_3gpp_flag;
+static gboolean enable_agps_flag;
+static gboolean disable_agps_flag;
 static gboolean enable_gps_nmea_flag;
 static gboolean disable_gps_nmea_flag;
 static gboolean get_gps_nmea_flag;
@@ -60,7 +62,11 @@ static gboolean disable_cdma_bs_flag;
 static gboolean get_cdma_bs_flag;
 static gboolean enable_gps_unmanaged_flag;
 static gboolean disable_gps_unmanaged_flag;
+static gboolean set_enable_signal_flag;
+static gboolean set_disable_signal_flag;
 static gboolean get_all_flag;
+static gchar *set_supl_server_str;
+static gchar *set_gps_refresh_rate_str;
 
 static GOptionEntry entries[] = {
     { "location-status", 0, 0, G_OPTION_ARG_NONE, &status_flag,
@@ -83,6 +89,14 @@ static GOptionEntry entries[] = {
       "Get 3GPP-based location.",
       NULL
     },
+    { "location-enable-agps", 0, 0, G_OPTION_ARG_NONE, &enable_agps_flag,
+      "Enable A-GPS location gathering.",
+      NULL
+    },
+    { "location-disable-agps", 0, 0, G_OPTION_ARG_NONE, &disable_agps_flag,
+      "Disable A-GPS location gathering.",
+      NULL
+    },
     { "location-enable-gps-nmea", 0, 0, G_OPTION_ARG_NONE, &enable_gps_nmea_flag,
       "Enable NMEA-based GPS location gathering.",
       NULL
@@ -127,6 +141,22 @@ static GOptionEntry entries[] = {
       "Disable unmanaged GPS location gathering.",
       NULL
     },
+    { "location-set-supl-server", 0, 0, G_OPTION_ARG_STRING, &set_supl_server_str,
+      "Set SUPL server address",
+      "[IP:PORT] or [URL]"
+    },
+    { "location-set-gps-refresh-rate", 0, 0, G_OPTION_ARG_STRING, &set_gps_refresh_rate_str,
+      "Set GPS refresh rate in seconds, or 0 disable the explicit rate.",
+      "[RATE]"
+    },
+    { "location-set-enable-signal", 0, 0, G_OPTION_ARG_NONE, &set_enable_signal_flag,
+      "Enable location update signaling in DBus property.",
+      NULL
+    },
+    { "location-set-disable-signal", 0, 0, G_OPTION_ARG_NONE, &set_disable_signal_flag,
+      "Disable location update signaling in DBus property.",
+      NULL
+    },
     { NULL }
 };
 
@@ -155,6 +185,7 @@ mmcli_modem_location_options_enabled (void)
         return !!n_actions;
 
     if ((enable_3gpp_flag && disable_3gpp_flag) ||
+        (enable_agps_flag && disable_agps_flag) ||
         (enable_gps_nmea_flag && disable_gps_nmea_flag) ||
         (enable_gps_raw_flag && disable_gps_raw_flag) ||
         (enable_gps_unmanaged_flag && disable_gps_unmanaged_flag) ||
@@ -163,6 +194,11 @@ mmcli_modem_location_options_enabled (void)
         exit (EXIT_FAILURE);
     }
 
+    if (set_enable_signal_flag && set_disable_signal_flag) {
+        g_printerr ("error: cannot enable and disable location signaling\n");
+        exit (EXIT_FAILURE);
+    }
+
     if (get_all_flag) {
         get_3gpp_flag = TRUE;
         get_gps_nmea_flag = TRUE;
@@ -173,6 +209,8 @@ mmcli_modem_location_options_enabled (void)
     n_actions = (status_flag +
                  !!(enable_3gpp_flag +
                     disable_3gpp_flag +
+                    enable_agps_flag +
+                    disable_agps_flag +
                     enable_gps_nmea_flag +
                     disable_gps_nmea_flag +
                     enable_gps_raw_flag +
@@ -180,11 +218,15 @@ mmcli_modem_location_options_enabled (void)
                     enable_cdma_bs_flag +
                     disable_cdma_bs_flag +
                     enable_gps_unmanaged_flag +
-                    disable_gps_unmanaged_flag) +
+                    disable_gps_unmanaged_flag +
+                    set_enable_signal_flag +
+                    set_disable_signal_flag) +
                  !!(get_3gpp_flag +
                     get_gps_nmea_flag +
                     get_gps_raw_flag +
-                    get_cdma_bs_flag));
+                    get_cdma_bs_flag) +
+                 !!set_supl_server_str +
+                 !!set_gps_refresh_rate_str);
 
     if (n_actions > 1) {
         g_printerr ("error: too many Location actions requested\n");
@@ -250,13 +292,37 @@ print_location_status (void)
     g_print ("\n"
              "%s\n"
              "  ----------------------------\n"
-             "  Location | capabilities: '%s'\n"
-             "           |      enabled: '%s'\n"
-             "           |      signals: '%s'\n",
+             "  Location |   capabilities: '%s'\n"
+             "           |        enabled: '%s'\n"
+             "           |        signals: '%s'\n",
              mm_modem_location_get_path (ctx->modem_location),
              capabilities_str,
              enabled_str,
              mm_modem_location_signals_location (ctx->modem_location) ? "yes" : "no");
+
+    /* If A-GPS supported, show SUPL server setup */
+    if (mm_modem_location_get_capabilities (ctx->modem_location) & MM_MODEM_LOCATION_SOURCE_AGPS) {
+        const gchar *supl_server;
+
+        supl_server = mm_modem_location_get_supl_server (ctx->modem_location);
+        g_print ("  ----------------------------\n"
+                 "  A-GPS    |    SUPL server: '%s'\n",
+                 supl_server ? supl_server : "unset");
+    }
+
+    /* If GPS supported, show GPS refresh rate */
+    if (mm_modem_location_get_capabilities (ctx->modem_location) & (MM_MODEM_LOCATION_SOURCE_GPS_RAW |
+                                                                    MM_MODEM_LOCATION_SOURCE_GPS_NMEA)) {
+        guint rate;
+
+        rate = mm_modem_location_get_gps_refresh_rate (ctx->modem_location);
+        g_print ("  ----------------------------\n");
+        if (rate > 0)
+            g_print ("  GPS      |  refresh rate: '%u'\n", rate);
+        else
+            g_print ("  GPS      |  refresh rate: disabled\n");
+    }
+
     g_free (capabilities_str);
     g_free (enabled_str);
 }
@@ -287,6 +353,58 @@ setup_ready (MMModemLocation *modem_location,
     mmcli_async_operation_done ();
 }
 
+static void
+set_supl_server_process_reply (gboolean result,
+                               const GError *error)
+{
+    if (!result) {
+        g_printerr ("error: couldn't set SUPL servert address: '%s'\n",
+                    error ? error->message : "unknown error");
+        exit (EXIT_FAILURE);
+    }
+
+    g_print ("successfully set SUPL server address\n");
+}
+
+static void
+set_supl_server_ready (MMModemLocation *modem_location,
+                       GAsyncResult    *result)
+{
+    gboolean operation_result;
+    GError *error = NULL;
+
+    operation_result = mm_modem_location_set_supl_server_finish (modem_location, result, &error);
+    set_supl_server_process_reply (operation_result, error);
+
+    mmcli_async_operation_done ();
+}
+
+static void
+set_gps_refresh_rate_process_reply (gboolean result,
+                                    const GError *error)
+{
+    if (!result) {
+        g_printerr ("error: couldn't set GPS refresh rate: '%s'\n",
+                    error ? error->message : "unknown error");
+        exit (EXIT_FAILURE);
+    }
+
+    g_print ("successfully set GPS refresh rate\n");
+}
+
+static void
+set_gps_refresh_rate_ready (MMModemLocation *modem_location,
+                            GAsyncResult    *result)
+{
+    gboolean operation_result;
+    GError *error = NULL;
+
+    operation_result = mm_modem_location_set_gps_refresh_rate_finish (modem_location, result, &error);
+    set_gps_refresh_rate_process_reply (operation_result, error);
+
+    mmcli_async_operation_done ();
+}
+
 static MMModemLocationSource
 build_sources_from_flags (void)
 {
@@ -300,6 +418,11 @@ build_sources_from_flags (void)
     if (disable_3gpp_flag)
         sources &= ~MM_MODEM_LOCATION_SOURCE_3GPP_LAC_CI;
 
+    if (enable_agps_flag)
+        sources |= MM_MODEM_LOCATION_SOURCE_AGPS;
+    if (disable_agps_flag)
+        sources &= ~MM_MODEM_LOCATION_SOURCE_AGPS;
+
     if (enable_gps_nmea_flag)
         sources |= MM_MODEM_LOCATION_SOURCE_GPS_NMEA;
     if (disable_gps_nmea_flag)
@@ -323,6 +446,16 @@ build_sources_from_flags (void)
     return sources;
 }
 
+static gboolean
+build_signals_location_from_flags (void)
+{
+    if (set_enable_signal_flag)
+        return TRUE;
+    if (set_disable_signal_flag)
+        return FALSE;
+    return mm_modem_location_signals_location (ctx->modem_location);
+}
+
 static void
 get_location_process_reply (MMLocation3gpp *location_3gpp,
                             MMLocationGpsNmea *location_gps_nmea,
@@ -486,6 +619,8 @@ get_modem_ready (GObject      *source,
     /* Request to setup location gathering? */
     if (enable_3gpp_flag ||
         disable_3gpp_flag ||
+        enable_agps_flag ||
+        disable_agps_flag ||
         enable_gps_nmea_flag ||
         disable_gps_nmea_flag ||
         enable_gps_raw_flag ||
@@ -493,11 +628,13 @@ get_modem_ready (GObject      *source,
         enable_cdma_bs_flag ||
         disable_cdma_bs_flag ||
         enable_gps_unmanaged_flag ||
-        disable_gps_unmanaged_flag) {
+        disable_gps_unmanaged_flag ||
+        set_enable_signal_flag ||
+        set_disable_signal_flag) {
         g_debug ("Asynchronously setting up location gathering...");
         mm_modem_location_setup (ctx->modem_location,
                                  build_sources_from_flags (),
-                                 mm_modem_location_signals_location (ctx->modem_location),
+                                 build_signals_location_from_flags (),
                                  ctx->cancellable,
                                  (GAsyncReadyCallback)setup_ready,
                                  NULL);
@@ -517,6 +654,35 @@ get_modem_ready (GObject      *source,
         return;
     }
 
+    /* Request to set SUPL server? */
+    if (set_supl_server_str) {
+        g_debug ("Asynchronously setting SUPL server...");
+        mm_modem_location_set_supl_server (ctx->modem_location,
+                                           set_supl_server_str,
+                                           ctx->cancellable,
+                                           (GAsyncReadyCallback)set_supl_server_ready,
+                                           NULL);
+        return;
+    }
+
+    /* Request to set GPS refresh rate? */
+    if (set_gps_refresh_rate_str) {
+        guint rate;
+
+        if (!mm_get_uint_from_str (set_gps_refresh_rate_str, &rate)) {
+            g_printerr ("error: couldn't set GPS refresh rate: invalid rate given: '%s'\n",
+                        set_gps_refresh_rate_str);
+            exit (EXIT_FAILURE);
+        }
+        g_debug ("Asynchronously setting GPS refresh rate...");
+        mm_modem_location_set_gps_refresh_rate (ctx->modem_location,
+                                                rate,
+                                                ctx->cancellable,
+                                                (GAsyncReadyCallback)set_gps_refresh_rate_ready,
+                                                NULL);
+        return;
+    }
+
     g_warn_if_reached ();
 }
 
@@ -565,6 +731,8 @@ mmcli_modem_location_run_synchronous (GDBusConnection *connection)
     /* Request to setup location gathering? */
     if (enable_3gpp_flag ||
         disable_3gpp_flag ||
+        enable_agps_flag ||
+        disable_agps_flag ||
         enable_gps_nmea_flag ||
         disable_gps_nmea_flag ||
         enable_gps_raw_flag ||
@@ -572,13 +740,15 @@ mmcli_modem_location_run_synchronous (GDBusConnection *connection)
         enable_cdma_bs_flag ||
         disable_cdma_bs_flag ||
         enable_gps_unmanaged_flag ||
-        disable_gps_unmanaged_flag) {
+        disable_gps_unmanaged_flag ||
+        set_enable_signal_flag ||
+        set_disable_signal_flag) {
         gboolean result;
 
         g_debug ("Synchronously setting up location gathering...");
         result = mm_modem_location_setup_sync (ctx->modem_location,
                                                build_sources_from_flags (),
-                                               mm_modem_location_signals_location (ctx->modem_location),
+                                               build_signals_location_from_flags (),
                                                NULL,
                                                &error);
         setup_process_reply (result, error);
@@ -607,5 +777,38 @@ mmcli_modem_location_run_synchronous (GDBusConnection *connection)
         return;
     }
 
+    /* Request to set SUPL server? */
+    if (set_supl_server_str) {
+        gboolean result;
+
+        g_debug ("Synchronously setting SUPL server...");
+        result = mm_modem_location_set_supl_server_sync (ctx->modem_location,
+                                                         set_supl_server_str,
+                                                         NULL,
+                                                         &error);
+        set_supl_server_process_reply (result, error);
+        return;
+    }
+
+    /* Request to set GPS refresh rate? */
+    if (set_gps_refresh_rate_str) {
+        gboolean result;
+        guint rate;
+
+        if (!mm_get_uint_from_str (set_gps_refresh_rate_str, &rate)) {
+            g_printerr ("error: couldn't set GPS refresh rate: invalid rate given: '%s'\n",
+                        set_gps_refresh_rate_str);
+            exit (EXIT_FAILURE);
+        }
+
+        g_debug ("Synchronously setting GPS refresh rate...");
+        result = mm_modem_location_set_gps_refresh_rate_sync (ctx->modem_location,
+                                                              rate,
+                                                              NULL,
+                                                              &error);
+        set_gps_refresh_rate_process_reply (result, error);
+        return;
+    }
+
     g_warn_if_reached ();
 }
