@@ -19,6 +19,9 @@
 
 #include <glib.h>
 
+/* Bands response parser */
+GArray *mm_altair_parse_bands_response (const gchar *response);
+
 /* +CEER response parser */
 gchar *mm_altair_parse_ceer_response (const gchar *response,
                                       GError **error);
@@ -27,8 +30,6 @@ gchar *mm_altair_parse_ceer_response (const gchar *response,
 guint mm_altair_parse_cid (const gchar *response, GError **error);
 
 /* %PCOINFO response parser */
-guint mm_altair_parse_vendor_pco_info (const gchar *pco_info,
-                                       guint cid,
-                                       GError **error);
+guint mm_altair_parse_vendor_pco_info (const gchar *pco_info, GError **error);
 
 #endif  /* MM_MODEM_HELPERS_ALTAIR_H */
