@@ -38,8 +38,8 @@
 
 G_DEFINE_TYPE (MMPluginHuawei, mm_plugin_huawei, MM_TYPE_PLUGIN)
 
-int mm_plugin_major_version = MM_PLUGIN_MAJOR_VERSION;
-int mm_plugin_minor_version = MM_PLUGIN_MINOR_VERSION;
+MM_PLUGIN_DEFINE_MAJOR_VERSION
+MM_PLUGIN_DEFINE_MINOR_VERSION
 
 /*****************************************************************************/
 /* Custom init */
@@ -159,6 +159,10 @@ getportmode_ready (MMPortSerialAt *port,
         cache_port_mode (device, response, "MDM:",  TAG_HUAWEI_MODEM_PORT);
         cache_port_mode (device, response, "NDIS:", TAG_HUAWEI_NDIS_PORT);
         cache_port_mode (device, response, "DIAG:", TAG_HUAWEI_DIAG_PORT);
+        /* GETPORTMODE response format in newer devices... (e.g. E3372) */
+        cache_port_mode (device, response, "pcui:",  TAG_HUAWEI_PCUI_PORT);
+        cache_port_mode (device, response, "modem:", TAG_HUAWEI_MODEM_PORT);
+
         g_object_set_data (G_OBJECT (device), TAG_GETPORTMODE_SUPPORTED, GUINT_TO_POINTER (TRUE));
 
         /* Mark port as being AT already */
@@ -324,7 +328,7 @@ first_interface_missing_timeout_cb (MMDevice *device)
 
     /* Reload the timeout, just in case we end up not having the next interface to probe...
      * which is anyway very unlikely as we got it by looking at the real probe list, but anyway... */
-    return TRUE;
+    return G_SOURCE_CONTINUE;
 }
 
 static void
