@@ -16,6 +16,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <locale.h>
+#include <arpa/inet.h>
 
 #include <ModemManager.h>
 #define _LIBMM_INSIDE_MM
@@ -138,6 +139,55 @@ test_ndisstatqry (void)
     }
 }
 
+/*****************************************************************************/
+/* Test ^DHCP responses */
+
+typedef struct {
+    const gchar *str;
+    const gchar *expected_addr;
+    guint expected_prefix;
+    const gchar *expected_gateway;
+    const gchar *expected_dns1;
+    const gchar *expected_dns2;
+} DhcpTest;
+
+static const DhcpTest dhcp_tests[] = {
+    { "^DHCP:a3ec5c64,f8ffffff,a1ec5c64,a1ec5c64,2200b10a,74bba80a,236800,236800\r\n",
+      "100.92.236.163", 29, "100.92.236.161", "10.177.0.34", "10.168.187.116" },
+    { "^DHCP: 1010A0A,FCFFFFFF,2010A0A,2010A0A,0,0,150000000,150000000\r\n",
+      "10.10.1.1", 30, "10.10.1.2", "0.0.0.0", "0.0.0.0" },
+    { "^DHCP: CCDB080A,F8FFFFFF,C9DB080A,C9DB080A,E67B59C0,E77B59C0,85600,85600\r\n",
+      "10.8.219.204", 29, "10.8.219.201", "192.89.123.230", "192.89.123.231" },
+    { NULL }
+};
+
+static void
+test_dhcp (void)
+{
+    guint i;
+
+    for (i = 0; dhcp_tests[i].str; i++) {
+        GError *error = NULL;
+        guint addr, prefix, gateway, dns1, dns2;
+
+        g_assert (mm_huawei_parse_dhcp_response (
+                      dhcp_tests[i].str,
+                      &addr,
+                      &prefix,
+                      &gateway,
+                      &dns1,
+                      &dns2,
+                      &error) == TRUE);
+        g_assert_no_error (error);
+
+        g_assert_cmpstr (inet_ntoa (*((struct in_addr *) &addr)), ==, dhcp_tests[i].expected_addr);
+        g_assert_cmpint (prefix, ==, dhcp_tests[i].expected_prefix);
+        g_assert_cmpstr (inet_ntoa (*((struct in_addr *) &gateway)), ==, dhcp_tests[i].expected_gateway);
+        g_assert_cmpstr (inet_ntoa (*((struct in_addr *) &dns1)), ==, dhcp_tests[i].expected_dns1);
+        g_assert_cmpstr (inet_ntoa (*((struct in_addr *) &dns2)), ==, dhcp_tests[i].expected_dns2);
+    }
+}
+
 /*****************************************************************************/
 /* Test ^SYSINFO responses */
 
@@ -424,6 +474,7 @@ test_prefmode_response (void)
         found = mm_huawei_parse_prefmode_response (prefmode_response_tests[i].str,
                                                    combinations,
                                                    &error);
+        g_assert_no_error (error);
         g_assert (found != NULL);
         g_assert_cmpuint (found->allowed, ==, prefmode_response_tests[i].allowed);
         g_assert_cmpuint (found->preferred, ==, prefmode_response_tests[i].preferred);
@@ -656,6 +707,20 @@ static const SyscfgResponseTest syscfg_response_tests[] = {
         .format = "^SYSCFG:(2,13,14,16),(0-3),((400000,\"WCDMA2100\")),(0-2),(0-4)\r\n",
         .allowed = MM_MODEM_MODE_3G,
         .preferred = MM_MODEM_MODE_NONE
+    },
+    {
+        /* Non-sensical acquisition order (WCDMA-only but acquire WCDMA-then-GSM */
+        .str = "^SYSCFG: 14,2,400000,0,3\r\n",
+        .format = "^SYSCFG:(2,13,14,16),(0-3),((400000,\"WCDMA2100\")),(0-2),(0-4)\r\n",
+        .allowed = MM_MODEM_MODE_3G,
+        .preferred = MM_MODEM_MODE_NONE
+    },
+    {
+        /* Non-sensical acquisition order (GSM-only but acquire GSM-then-WCDMA */
+        .str = "^SYSCFG: 13,1,400000,0,3\r\n",
+        .format = "^SYSCFG:(2,13,14,16),(0-3),((400000,\"WCDMA2100\")),(0-2),(0-4)\r\n",
+        .allowed = MM_MODEM_MODE_2G,
+        .preferred = MM_MODEM_MODE_NONE
     }
 };
 
@@ -676,6 +741,7 @@ test_syscfg_response (void)
                                                  combinations,
                                                  &error);
 
+        g_assert_no_error (error);
         g_assert (found != NULL);
         g_assert_cmpuint (found->allowed, ==, syscfg_response_tests[i].allowed);
         g_assert_cmpuint (found->preferred, ==, syscfg_response_tests[i].preferred);
@@ -997,6 +1063,7 @@ test_syscfgex_response (void)
                                                    combinations,
                                                    &error);
 
+        g_assert_no_error (error);
         g_assert (found != NULL);
         g_assert_cmpuint (found->allowed, ==, syscfgex_response_tests[i].allowed);
         g_assert_cmpuint (found->preferred, ==, syscfgex_response_tests[i].preferred);
@@ -1089,8 +1156,10 @@ test_nwtime (void)
             g_assert (nwtime_tests[i].leap_seconds == mm_network_timezone_get_leap_seconds (tz));
         }
 
-        if (iso8601)
-            g_free (iso8601);
+        g_free (iso8601);
+
+        if (tz)
+            g_object_unref (tz);
     }
 }
 
@@ -1129,11 +1198,10 @@ test_time (void)
 
         g_assert (ret == time_tests[i].ret);
         g_assert (ret == (error ? FALSE : TRUE));
+        g_clear_error (&error);
 
         g_assert_cmpstr (time_tests[i].iso8601, ==, iso8601);
-
-        if (iso8601)
-            g_free (iso8601);
+        g_free (iso8601);
     }
 }
 
@@ -1167,6 +1235,7 @@ int main (int argc, char **argv)
     g_test_init (&argc, &argv, NULL);
 
     g_test_add_func ("/MM/huawei/ndisstatqry", test_ndisstatqry);
+    g_test_add_func ("/MM/huawei/dhcp", test_dhcp);
     g_test_add_func ("/MM/huawei/sysinfo", test_sysinfo);
     g_test_add_func ("/MM/huawei/sysinfoex", test_sysinfoex);
     g_test_add_func ("/MM/huawei/prefmode", test_prefmode);
