@@ -1160,7 +1160,7 @@ periodic_registration_check (MMIfaceModemCdma *self)
             (GAsyncReadyCallback)periodic_registration_checks_ready,
             NULL);
     }
-    return TRUE;
+    return G_SOURCE_CONTINUE;
 }
 
 static void
@@ -1414,7 +1414,6 @@ typedef enum {
     ENABLING_STEP_FIRST,
     ENABLING_STEP_SETUP_UNSOLICITED_EVENTS,
     ENABLING_STEP_ENABLE_UNSOLICITED_EVENTS,
-    ENABLING_STEP_RUN_REGISTRATION_CHECKS,
     ENABLING_STEP_PERIODIC_REGISTRATION_CHECKS,
     ENABLING_STEP_LAST
 } EnablingStep;
@@ -1499,24 +1498,6 @@ enable_unsolicited_events_ready (MMIfaceModemCdma *self,
     interface_enabling_step (ctx);
 }
 
-static void
-run_registration_checks_ready (MMIfaceModemCdma *self,
-                                   GAsyncResult *res,
-                                   EnablingContext *ctx)
-{
-    GError *error = NULL;
-
-    if (!mm_iface_modem_cdma_run_registration_checks_finish (self, res, &error)) {
-        g_simple_async_result_take_error (ctx->result, error);
-        enabling_context_complete_and_free (ctx);
-        return;
-    }
-
-    /* Go on to next step */
-    ctx->step++;
-    interface_enabling_step (ctx);
-}
-
 static void
 interface_enabling_step (EnablingContext *ctx)
 {
@@ -1553,12 +1534,6 @@ interface_enabling_step (EnablingContext *ctx)
         /* Fall down to next step */
         ctx->step++;
 
-    case ENABLING_STEP_RUN_REGISTRATION_CHECKS:
-        mm_iface_modem_cdma_run_registration_checks (ctx->self,
-                                                     (GAsyncReadyCallback)run_registration_checks_ready,
-                                                     ctx);
-        return;
-
     case ENABLING_STEP_PERIODIC_REGISTRATION_CHECKS:
         periodic_registration_check_enable (ctx->self);
         /* Fall down to next step */
